@@ -3,10 +3,14 @@ import functools
 import inspect
 import sys
 import time
+import calendar
 
 from dateutil import parser
 
 real_time = time.time
+real_localtime = time.localtime
+real_gmtime = time.gmtime
+real_strftime = time.strftime
 real_date = datetime.date
 real_datetime = datetime.datetime
 
@@ -28,8 +32,39 @@ class FakeTime(object):
         self.previous_time_function = previous_time_function
 
     def __call__(self):
+        return calendar.timegm(self.time_to_freeze.timetuple()) + self.time_to_freeze.microsecond / 1000000.0
+
+
+class FakeLocalTime(object):
+    def __init__(self, time_to_freeze):
+        self.time_to_freeze = time_to_freeze
+
+    def __call__(self, t=None):
+        if t is not None:
+            return real_localtime(t)
         shifted_time = self.time_to_freeze - datetime.timedelta(seconds=time.timezone)
-        return time.mktime(shifted_time.timetuple()) + shifted_time.microsecond / 1000000.0
+        return shifted_time.timetuple()
+
+
+class FakeGMTTime(object):
+    def __init__(self, time_to_freeze, previous_gmtime_function):
+        self.time_to_freeze = time_to_freeze
+        self.previous_gmtime_function = previous_gmtime_function
+
+    def __call__(self, t=None):
+        if t is not None:
+            return real_gmtime(t)
+        return self.time_to_freeze.timetuple()
+
+
+class FakeStrfTime(object):
+    def __init__(self, default_fake_time):
+        self.default_fake_time = default_fake_time
+
+    def __call__(self, format, time_to_format=None):
+        if time_to_format is None:
+            time_to_format = FakeLocalTime(self.default_fake_time)()
+        return real_strftime(format, time_to_format)
 
 
 class FakeDateMeta(type):
@@ -128,6 +163,9 @@ class FakeDatetime(with_metaclass(FakeDatetimeMeta, real_datetime, FakeDate)):
             result = cls._time_to_freeze() + datetime.timedelta(hours=cls._tz_offset())
         return datetime_to_fakedatetime(result)
 
+    def date(self):
+        return date_to_fakedate(self)
+
     @classmethod
     def today(cls):
         return cls.now(tz=None)
@@ -182,7 +220,12 @@ def pickle_fake_datetime(datetime_):
 class _freeze_time(object):
 
     def __init__(self, time_to_freeze_str, tz_offset, ignore):
-        time_to_freeze = parser.parse(time_to_freeze_str)
+        if isinstance(time_to_freeze_str, datetime.datetime):
+            time_to_freeze = time_to_freeze_str
+        elif isinstance(time_to_freeze_str, datetime.date):
+            time_to_freeze = datetime.datetime.combine(time_to_freeze_str, datetime.time())
+        else:
+            time_to_freeze = parser.parse(time_to_freeze_str)
         time_to_freeze = convert_to_timezone_naive(time_to_freeze)
 
         self.time_to_freeze = time_to_freeze
@@ -196,23 +239,21 @@ class _freeze_time(object):
         return self.decorate_callable(func)
 
     def decorate_class(self, klass):
-        for attr in dir(klass):
-            if attr.startswith("_"):
-                continue
+        seen = set()
+        for base_klass in klass.mro():
+            for (attr, attr_value) in base_klass.__dict__.items():
+                if attr.startswith('_') or attr in seen:
+                    continue
+                seen.add(attr)
 
-            attr_value = getattr(klass, attr)
-            if not hasattr(attr_value, "__call__"):
-                continue
+                if not callable(attr_value) or inspect.isclass(attr_value):
+                    continue
 
-            # Check if this is a classmethod. If so, skip patching
-            if inspect.ismethod(attr_value) and attr_value.__self__ is klass:
-                continue
-
-            try:
-                setattr(klass, attr, self(attr_value))
-            except TypeError:
-                # Sometimes we can't set this for built-in types
-                continue
+                try:
+                    setattr(klass, attr, self(attr_value))
+                except (AttributeError, TypeError):
+                    # Sometimes we can't set this for built-in types and custom callables
+                    continue
         return klass
 
     def __enter__(self):
@@ -226,7 +267,13 @@ class _freeze_time(object):
         datetime.datetime = FakeDatetime
         datetime.date = FakeDate
         fake_time = FakeTime(self.time_to_freeze, time.time)
+        fake_localtime = FakeLocalTime(self.time_to_freeze)
+        fake_gmtime = FakeGMTTime(self.time_to_freeze, time.gmtime)
+        fake_strftime = FakeStrfTime(self.time_to_freeze)
         time.time = fake_time
+        time.localtime = fake_localtime
+        time.gmtime = fake_gmtime
+        time.strftime = fake_strftime
 
         copyreg.dispatch_table[real_datetime] = pickle_fake_datetime
         copyreg.dispatch_table[real_date] = pickle_fake_date
@@ -241,7 +288,8 @@ class _freeze_time(object):
                 or module.__name__ in ['datetime', 'time']):
                 continue
             for module_attribute in dir(module):
-                if module_attribute in ['real_date', 'real_datetime', 'real_time']:
+                if module_attribute in ['real_date', 'real_datetime',
+                    'real_time', 'real_localtime', 'real_gmtime', 'real_strftime']:
                     continue
                 try:
                     attribute_value = getattr(module, module_attribute)
@@ -258,6 +306,15 @@ class _freeze_time(object):
                     elif attribute_value is real_time:
                         setattr(module, module_attribute, fake_time)
                         self.undo_changes.append((module, module_attribute, real_time))
+                    elif attribute_value is real_localtime:
+                        setattr(module, module_attribute, fake_localtime)
+                        self.undo_changes.append((module, module_attribute, real_localtime))
+                    elif attribute_value is real_gmtime:
+                        setattr(module, module_attribute, fake_gmtime)
+                        self.undo_changes.append((module, module_attribute, real_gmtime))
+                    elif attribute_value is real_strftime:
+                        setattr(module, module_attribute, fake_strftime)
+                        self.undo_changes.append((module, module_attribute, real_strftime))
                 except:
                     # If it's not possible to compare the value to real_XXX (e.g. hiredis.version)
                     pass
@@ -272,19 +329,18 @@ class _freeze_time(object):
     def stop(self):
         datetime.datetime.times_to_freeze.pop()
         datetime.datetime.tz_offsets.pop()
+        datetime.date.dates_to_freeze.pop()
+
         if not datetime.datetime.times_to_freeze:
             datetime.datetime = real_datetime
-            copyreg.dispatch_table.pop(real_datetime)
-
-        datetime.date.dates_to_freeze.pop()
-        if not datetime.date.dates_to_freeze:
             datetime.date = real_date
+            copyreg.dispatch_table.pop(real_datetime)
             copyreg.dispatch_table.pop(real_date)
+            for module, module_attribute, original_value in self.undo_changes:
+                setattr(module, module_attribute, original_value)
 
         time.time = time.time.previous_time_function
-
-        for module, module_attribute, original_value in self.undo_changes:
-            setattr(module, module_attribute, original_value)
+        time.gmtime = time.gmtime.previous_gmtime_function
 
     def decorate_callable(self, func):
         def wrapper(*args, **kwargs):
@@ -301,24 +357,19 @@ class _freeze_time(object):
 
 
 def freeze_time(time_to_freeze, tz_offset=0, ignore=None):
-    if isinstance(time_to_freeze, datetime.datetime):
-        time_to_freeze = time_to_freeze.isoformat()
-    elif isinstance(time_to_freeze, datetime.date):
-        time_to_freeze = time_to_freeze.isoformat()
-
     # Python3 doesn't have basestring, but it does have str.
     try:
         string_type = basestring
     except NameError:
         string_type = str
 
-    if not isinstance(time_to_freeze, string_type):
+    if not isinstance(time_to_freeze, (string_type, datetime.date)):
         raise TypeError(('freeze_time() expected a string, date instance, or '
                          'datetime instance, but got type {0}.').format(type(time_to_freeze)))
     if ignore is None:
         ignore = []
-    ignore.append('six.moves.')
-    ignore.append('django.utils.six.moves.')
+    ignore.append('six.moves')
+    ignore.append('django.utils.six.moves')
     return _freeze_time(time_to_freeze, tz_offset, ignore)
 
 
